/*
 * Titanium SDK
 * Copyright TiDev, Inc. 04/07/2022-Present. All Rights Reserved.
 * Licensed under the terms of the Apache Public License
 *
 */
/* eslint-env mocha */
/* eslint no-unused-expressions: "off" */
/* eslint no-undef: "off" */
'use strict';

const should = require('./utilities/assertions');

describe.android('Titanium.Android.R', () => {
	describe('properties', () => {
		describe('.anim', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('anim').which.is.a.Object();
			});

			const properties = [
				'accelerate_decelerate_interpolator',
				'accelerate_interpolator',
				'anticipate_interpolator',
				'anticipate_overshoot_interpolator',
				'bounce_interpolator',
				'cycle_interpolator',
				'decelerate_interpolator',
				'fade_in',
				'fade_out',
				'linear_interpolator',
				'overshoot_interpolator',
				'slide_in_left',
				'slide_out_right',
			];
			for (const prop of properties) {
				it(`.${prop} is a Number`, () => {
					should(Ti.Android.R.anim[prop]).be.a.Number();
					should(Ti.Android.R.anim[prop]).not.eql(0);
				});
			}
		});

		describe('.animator', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('animator').which.is.a.Object();
			});

			const properties = [
				'fade_in',
				'fade_out',
			];
			for (const prop of properties) {
				it(`.${prop} is a Number`, () => should(Ti.Android.R.animator[prop]).be.a.Number());
			}
		});

		describe('.apiName', () => {
			it('is a String', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('apiName').which.is.a.String();
			});

			it('equals Ti.Android.R', () => {
				should(Ti.Android.R.apiName).eql('Ti.Android.R');
			});
		});

		describe('.array', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('array').which.is.a.Object();
			});

			const properties = [
				'emailAddressTypes',
				'imProtocols',
				'organizationTypes',
				'phoneTypes',
				'postalAddressTypes',
			];
			for (const prop of properties) {
				it(`.${prop} is a Number`, () => {
					should(Ti.Android.R.array[prop]).be.a.Number();
					should(Ti.Android.R.array[prop]).not.eql(0);
				});
			}
		});

		describe('.attr', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('attr').which.is.a.Object();
			});

			const properties = [
				'absListViewStyle',
				[ 'accessibilityEventTypes', 14 ],
				'accessibilityFeedbackType',
				'accessibilityFlags',
				[ 'accessibilityHeading', 28 ],
				[ 'accessibilityLiveRegion', 19 ],
				[ 'accessibilityPaneTitle', 28 ],
				[ 'accessibilityTraversalAfter', 22 ],
				[ 'accessibilityTraversalBefore', 22 ],
				[ 'accountPreferences', 5 ],
				'accountType',
				'action',
				[ 'actionBarDivider', 14 ],
				[ 'actionBarItemBackground', 14 ],
				[ 'actionBarPopupTheme', 21 ],
				[ 'actionBarSize', 11 ],
				[ 'actionBarSplitStyle', 14 ],
				[ 'actionBarStyle', 11 ],
				[ 'actionBarTabBarStyle', 11 ],
				[ 'actionBarTabStyle', 11 ],
				[ 'actionBarTabTextStyle', 11 ],
				[ 'actionBarTheme', 21 ],
				[ 'actionBarWidgetTheme', 14 ],
				[ 'actionButtonStyle', 11 ],
				[ 'actionDropDownStyle', 11 ],
				[ 'actionLayout', 11 ],
				[ 'actionMenuTextAppearance', 11 ],
				[ 'actionMenuTextColor', 11 ],
				[ 'actionModeBackground', 11 ],
				[ 'actionModeCloseButtonStyle', 11 ],
				[ 'actionModeCloseDrawable', 11 ],
				[ 'actionModeCopyDrawable', 11 ],
				[ 'actionModeCutDrawable', 11 ],
				[ 'actionModeFindDrawable', 21 ],
				[ 'actionModePasteDrawable', 11 ],
				[ 'actionModeSelectAllDrawable', 14 ],
				[ 'actionModeShareDrawable', 21 ],
				[ 'actionModeSplitBackground', 14 ],
				[ 'actionModeStyle', 14 ],
				[ 'actionModeWebSearchDrawable', 21 ],
				[ 'actionOverflowButtonStyle', 11 ],
				[ 'actionOverflowMenuStyle', 21 ],
				[ 'actionProviderClass', 14 ],
				[ 'actionViewClass', 11 ],
				[ 'activatedBackgroundIndicator', 11 ],
				'activityCloseEnterAnimation',
				'activityCloseExitAnimation',
				'activityOpenEnterAnimation',
				'activityOpenExitAnimation',
				[ 'addPrintersActivity', 19 ],
				'addStatesFromChildren',
				'adjustViewBounds',
				[ 'advancedPrintOptionsActivity', 19 ],
				[ 'alertDialogIcon', 11 ],
				'alertDialogStyle',
				[ 'alertDialogTheme', 11 ],
				[ 'alignmentMode', 14 ],
				[ 'allContactsName', 11 ],
				[ 'allowAudioPlaybackCapture', 29 ],
				'allowBackup',
				'allowClearUserData',
				[ 'allowEmbedded', 20 ],
				[ 'allowNativeHeapPointerTagging', 30 ],
				[ 'allowParallelSyncs', 11 ],
				'allowSingleTap',
				'allowTaskReparenting',
				[ 'allowUndo', 23 ],
				[ 'alpha', 11 ],
				[ 'alphabeticModifiers', 26 ],
				'alphabeticShortcut',
				'alwaysDrawnWithCache',
				'alwaysRetainTaskState',
				[ 'amPmBackgroundColor', 21 ],
				[ 'amPmTextColor', 21 ],
				[ 'ambientShadowAlpha', 21 ],
				'angle',
				[ 'animateFirstView', 11 ],
				[ 'animateLayoutChanges', 11 ],
				'animateOnClick',
				[ 'animatedImageDrawable', 30 ],
				'animation',
				'animationCache',
				'animationDuration',
				'animationOrder',
				[ 'animationResolution', 11 ],
				'antialias',
				'anyDensity',
				[ 'apduServiceBanner', 19 ],
				'apiKey',
				[ 'appCategory', 26 ],
				[ 'appComponentFactory', 28 ],
				'author',
				'authorities',
				[ 'autoAdvanceViewId', 11 ],
				'autoCompleteTextViewStyle',
				'autoLink',
				[ 'autoMirrored', 19 ],
				[ 'autoRemoveFromRecents', 21 ],
				[ 'autoRevokePermissions', 30 ],
				[ 'autoSizeMaxTextSize', 26 ],
				[ 'autoSizeMinTextSize', 26 ],
				[ 'autoSizePresetSizes', 26 ],
				[ 'autoSizeStepGranularity', 26 ],
				[ 'autoSizeTextType', 26 ],
				'autoStart',
				'autoText',
				'autoUrlDetect',
				[ 'autoVerify', 23 ],
				[ 'autofillHints', 26 ],
				[ 'autofilledHighlight', 26 ],
				'background',
				'backgroundDimAmount',
				'backgroundDimEnabled',
				[ 'backgroundSplit', 14 ],
				[ 'backgroundStacked', 14 ],
				[ 'backgroundTint', 21 ],
				[ 'backgroundTintMode', 21 ],
				'backupAgent',
				[ 'backupInForeground', 24 ],
				[ 'banner', 21 ],
				[ 'baseline', 11 ],
				'baselineAlignBottom',
				'baselineAligned',
				'baselineAlignedChildIndex',
				[ 'bitmap', 24 ],
				[ 'borderlessButtonStyle', 11 ],
				'bottom',
				'bottomBright',
				'bottomDark',
				'bottomLeftRadius',
				'bottomMedium',
				'bottomOffset',
				'bottomRightRadius',
				'breadCrumbShortTitle',
				'breadCrumbTitle',
				[ 'breakStrategy', 23 ],
				'bufferType',
				'button',
				'buttonBarButtonStyle',
				[ 'buttonBarNegativeButtonStyle', 21 ],
				[ 'buttonBarNeutralButtonStyle', 21 ],
				[ 'buttonBarPositiveButtonStyle', 21 ],
				[ 'buttonBarStyle', 11 ],
				[ 'buttonCornerRadius', 28 ],
				[ 'buttonGravity', 24 ],
				'buttonStyle',
				'buttonStyleInset',
				'buttonStyleSmall',
				'buttonStyleToggle',
				[ 'buttonTint', 21 ],
				[ 'buttonTintMode', 21 ],
				'cacheColorHint',
				[ 'calendarTextColor', 21 ],
				[ 'calendarViewShown', 11 ],
				[ 'calendarViewStyle', 11 ],
				[ 'canControlMagnification', 24 ],
				[ 'canPerformGestures', 24 ],
				[ 'canRecord', 24 ],
				[ 'canRequestEnhancedWebAccessibility', 18 ],
				[ 'canRequestFilterKeyEvents', 18 ],
				[ 'canRequestFingerprintGestures', 26 ],
				[ 'canRequestTouchExplorationMode', 18 ],
				[ 'canRetrieveWindowContent', 14 ],
				[ 'canTakeScreenshot', 30 ],
				'candidatesTextStyleSpans',
				[ 'cantSaveState', 28 ],
				'capitalize',
				[ 'category', 19 ],
				'centerBright',
				'centerColor',
				'centerDark',
				'centerMedium',
				'centerX',
				'centerY',
				[ 'certDigest', 26 ],
				'checkBoxPreferenceStyle',
				'checkMark',
				[ 'checkMarkTint', 21 ],
				[ 'checkMarkTintMode', 21 ],
				'checkable',
				'checkableBehavior',
				'checkboxStyle',
				'checked',
				'checkedButton',
				[ 'checkedTextViewStyle', 17 ],
				'childDivider',
				'childIndicator',
				[ 'childIndicatorEnd', 18 ],
				'childIndicatorLeft',
				'childIndicatorRight',
				[ 'childIndicatorStart', 18 ],
				'choiceMode',
				[ 'classLoader', 27 ],
				'clearTaskOnLaunch',
				'clickable',
				'clipChildren',
				'clipOrientation',
				'clipToPadding',
				[ 'closeIcon', 21 ],
				'codes',
				'collapseColumns',
				[ 'collapseContentDescription', 22 ],
				[ 'collapseIcon', 24 ],
				'color',
				[ 'colorAccent', 21 ],
				[ 'colorActivatedHighlight', 14 ],
				'colorBackground',
				'colorBackgroundCacheHint',
				[ 'colorBackgroundFloating', 23 ],
				[ 'colorButtonNormal', 21 ],
				[ 'colorControlActivated', 21 ],
				[ 'colorControlHighlight', 21 ],
				[ 'colorControlNormal', 21 ],
				[ 'colorEdgeEffect', 21 ],
				[ 'colorError', 26 ],
				[ 'colorFocusedHighlight', 14 ],
				'colorForeground',
				'colorForegroundInverse',
				[ 'colorLongPressedHighlight', 14 ],
				[ 'colorMode', 26 ],
				[ 'colorMultiSelectHighlight', 14 ],
				[ 'colorPressedHighlight', 14 ],
				[ 'colorPrimary', 21 ],
				[ 'colorPrimaryDark', 21 ],
				[ 'colorSecondary', 25 ],
				[ 'columnCount', 14 ],
				'columnDelay',
				[ 'columnOrderPreserved', 14 ],
				'columnWidth',
				[ 'commitIcon', 21 ],
				[ 'compatibleWidthLimitDp', 13 ],
				'completionHint',
				'completionHintView',
				'completionThreshold',
				'configChanges',
				'configure',
				'constantSize',
				'content',
				[ 'contentAgeHint', 21 ],
				'contentAuthority',
				'contentDescription',
				[ 'contentInsetEnd', 21 ],
				[ 'contentInsetEndWithActions', 24 ],
				[ 'contentInsetLeft', 21 ],
				[ 'contentInsetRight', 21 ],
				[ 'contentInsetStart', 21 ],
				[ 'contentInsetStartWithNavigation', 24 ],
				[ 'contextClickable', 23 ],
				[ 'contextDescription', 25 ],
				[ 'contextPopupMenuStyle', 24 ],
				[ 'contextUri', 25 ],
				[ 'controlX1', 21 ],
				[ 'controlX2', 21 ],
				[ 'controlY1', 21 ],
				[ 'controlY2', 21 ],
				[ 'countDown', 24 ],
				[ 'country', 21 ],
				'cropToPadding',
				[ 'crossProfile', 30 ],
				'cursorVisible',
				[ 'customNavigationLayout', 11 ],
				[ 'customTokens', 11 ],
				'cycles',
				'dashGap',
				'dashWidth',
				'data',
				[ 'datePickerDialogTheme', 21 ],
				[ 'datePickerMode', 21 ],
				[ 'datePickerStyle', 11 ],
				[ 'dateTextAppearance', 11 ],
				[ 'dayOfWeekBackground', 21 ],
				[ 'dayOfWeekTextAppearance', 21 ],
				'debuggable',
				[ 'defaultFocusHighlightEnabled', 26 ],
				[ 'defaultHeight', 24 ],
				[ 'defaultToDeviceProtectedStorage', 24 ],
				'defaultValue',
				[ 'defaultWidth', 24 ],
				'delay',
				'dependency',
				'descendantFocusability',
				'description',
				'detachWallpaper',
				'detailColumn',
				'detailSocialSummary',
				'detailsElementBackground',
				'dial',
				[ 'dialogCornerRadius', 28 ],
				'dialogIcon',
				'dialogLayout',
				'dialogMessage',
				'dialogPreferenceStyle',
				[ 'dialogPreferredPadding', 22 ],
				[ 'dialogTheme', 11 ],
				'dialogTitle',
				'digits',
				[ 'directBootAware', 24 ],
				'direction',
				[ 'directionDescriptions', 14 ],
				'directionPriority',
				'disableDependentsState',
				'disabledAlpha',
				[ 'displayOptions', 11 ],
				'dither',
				'divider',
				'dividerHeight',
				[ 'dividerHorizontal', 11 ],
				[ 'dividerPadding', 11 ],
				[ 'dividerVertical', 11 ],
				[ 'documentLaunchMode', 21 ],
				'drawSelectorOnTop',
				'drawable',
				'drawableBottom',
				[ 'drawableEnd', 14 ],
				'drawableLeft',
				'drawablePadding',
				'drawableRight',
				[ 'drawableStart', 14 ],
				[ 'drawableTint', 23 ],
				[ 'drawableTintMode', 23 ],
				'drawableTop',
				'drawingCacheQuality',
				'dropDownAnchor',
				'dropDownHeight',
				'dropDownHintAppearance',
				'dropDownHorizontalOffset',
				'dropDownItemStyle',
				'dropDownListViewStyle',
				'dropDownSelector',
				[ 'dropDownSpinnerStyle', 11 ],
				'dropDownVerticalOffset',
				'dropDownWidth',
				'duplicateParentState',
				'duration',
				[ 'editTextBackground', 11 ],
				[ 'editTextColor', 11 ],
				'editTextPreferenceStyle',
				'editTextStyle',
				'editable',
				'editorExtras',
				[ 'elegantTextHeight', 21 ],
				[ 'elevation', 21 ],
				'ellipsize',
				'ems',
				[ 'enableVrMode', 24 ],
				'enabled',
				[ 'end', 23 ],
				'endColor',
				[ 'endX', 24 ],
				[ 'endY', 24 ],
				'endYear',
				[ 'enforceNavigationBarContrast', 29 ],
				[ 'enforceStatusBarContrast', 29 ],
				[ 'enterFadeDuration', 11 ],
				'entries',
				'entryValues',
				'eventsInterceptionEnabled',
				[ 'excludeClass', 21 ],
				'excludeFromRecents',
				[ 'excludeId', 21 ],
				[ 'excludeName', 21 ],
				[ 'exitFadeDuration', 11 ],
				'expandableListPreferredChildIndicatorLeft',
				'expandableListPreferredChildIndicatorRight',
				'expandableListPreferredChildPaddingLeft',
				'expandableListPreferredItemIndicatorLeft',
				'expandableListPreferredItemIndicatorRight',
				'expandableListPreferredItemPaddingLeft',
				'expandableListViewStyle',
				'expandableListViewWhiteStyle',
				'exported',
				[ 'externalService', 24 ],
				'extraTension',
				[ 'extractNativeLibs', 23 ],
				'factor',
				'fadeDuration',
				'fadeEnabled',
				'fadeOffset',
				'fadeScrollbars',
				'fadingEdge',
				'fadingEdgeLength',
				[ 'fadingMode', 19 ],
				[ 'fallbackLineSpacing', 28 ],
				[ 'fastScrollAlwaysVisible', 11 ],
				'fastScrollEnabled',
				[ 'fastScrollOverlayPosition', 11 ],
				[ 'fastScrollPreviewBackgroundLeft', 11 ],
				[ 'fastScrollPreviewBackgroundRight', 11 ],
				[ 'fastScrollStyle', 21 ],
				[ 'fastScrollTextColor', 11 ],
				[ 'fastScrollThumbDrawable', 11 ],
				[ 'fastScrollTrackDrawable', 11 ],
				'fillAfter',
				[ 'fillAlpha', 21 ],
				'fillBefore',
				[ 'fillColor', 21 ],
				'fillEnabled',
				[ 'fillType', 24 ],
				'fillViewport',
				'filter',
				'filterTouchesWhenObscured',
				[ 'fingerprintAuthDrawable', 23 ],
				'finishOnCloseSystemDialogs',
				'finishOnTaskLaunch',
				[ 'firstBaselineToTopHeight', 28 ],
				[ 'firstDayOfWeek', 11 ],
				'fitsSystemWindows',
				'flipInterval',
				'focusable',
				'focusableInTouchMode',
				[ 'focusedByDefault', 26 ],
				[ 'focusedMonthDateColor', 11 ],
				[ 'font', 26 ],
				[ 'fontFamily', 16 ],
				[ 'fontFeatureSettings', 21 ],
				[ 'fontProviderAuthority', 26 ],
				[ 'fontProviderCerts', 26 ],
				[ 'fontProviderPackage', 26 ],
				[ 'fontProviderQuery', 26 ],
				[ 'fontStyle', 26 ],
				[ 'fontVariationSettings', 28 ],
				[ 'fontWeight', 26 ],
				'footerDividersEnabled',
				[ 'forceDarkAllowed', 29 ],
				[ 'forceHasOverlappingRendering', 24 ],
				[ 'forceQueryable', 30 ],
				[ 'forceUriPermissions', 29 ],
				'foreground',
				'foregroundGravity',
				[ 'foregroundServiceType', 29 ],
				[ 'foregroundTint', 21 ],
				[ 'foregroundTintMode', 21 ],
				'format',
				[ 'format12Hour', 17 ],
				[ 'format24Hour', 17 ],
				[ 'fraction', 23 ],
				[ 'fragment', 11 ],
				[ 'fragmentAllowEnterTransitionOverlap', 21 ],
				[ 'fragmentAllowReturnTransitionOverlap', 21 ],
				[ 'fragmentCloseEnterAnimation', 11 ],
				[ 'fragmentCloseExitAnimation', 11 ],
				[ 'fragmentEnterTransition', 21 ],
				[ 'fragmentExitTransition', 21 ],
				[ 'fragmentFadeEnterAnimation', 11 ],
				[ 'fragmentFadeExitAnimation', 11 ],
				[ 'fragmentOpenEnterAnimation', 11 ],
				[ 'fragmentOpenExitAnimation', 11 ],
				[ 'fragmentReenterTransition', 21 ],
				[ 'fragmentReturnTransition', 21 ],
				[ 'fragmentSharedElementEnterTransition', 21 ],
				[ 'fragmentSharedElementReturnTransition', 21 ],
				'freezesText',
				'fromAlpha',
				'fromDegrees',
				[ 'fromId', 21 ],
				[ 'fromScene', 19 ],
				'fromXDelta',
				'fromXScale',
				'fromYDelta',
				'fromYScale',
				[ 'fullBackupContent', 23 ],
				[ 'fullBackupOnly', 21 ],
				'fullBright',
				'fullDark',
				'functionalTest',
				'galleryItemBackground',
				'galleryStyle',
				'gestureColor',
				'gestureStrokeAngleThreshold',
				'gestureStrokeLengthThreshold',
				'gestureStrokeSquarenessThreshold',
				'gestureStrokeType',
				'gestureStrokeWidth',
				'glEsVersion',
				[ 'goIcon', 21 ],
				'gradientRadius',
				'grantUriPermissions',
				'gravity',
				'gridViewStyle',
				'groupIndicator',
				[ 'gwpAsanMode', 30 ],
				'hand_hour',
				'hand_minute',
				'handle',
				'handleProfiling',
				'hapticFeedbackEnabled',
				[ 'hardwareAccelerated', 11 ],
				'hasCode',
				[ 'hasFragileUserData', 29 ],
				[ 'headerAmPmTextAppearance', 21 ],
				'headerBackground',
				[ 'headerDayOfMonthTextAppearance', 21 ],
				'headerDividersEnabled',
				[ 'headerMonthTextAppearance', 21 ],
				[ 'headerTimeTextAppearance', 21 ],
				[ 'headerYearTextAppearance', 21 ],
				'height',
				[ 'hideOnContentScroll', 21 ],
				'hint',
				[ 'homeAsUpIndicator', 11 ],
				[ 'homeLayout', 11 ],
				'horizontalDivider',
				'horizontalGap',
				[ 'horizontalScrollViewStyle', 11 ],
				'horizontalSpacing',
				'host',
				[ 'hotSpotX', 24 ],
				[ 'hotSpotY', 24 ],
				[ 'htmlDescription', 30 ],
				[ 'hyphenationFrequency', 23 ],
				'icon',
				'iconPreview',
				[ 'iconSpaceReserved', 26 ],
				[ 'iconTint', 26 ],
				[ 'iconTintMode', 26 ],
				[ 'iconifiedByDefault', 11 ],
				'id',
				[ 'identifier', 29 ],
				'ignoreGravity',
				'imageButtonStyle',
				'imageWellStyle',
				'imeActionId',
				'imeActionLabel',
				'imeExtractEnterAnimation',
				'imeExtractExitAnimation',
				'imeFullscreenBackground',
				'imeOptions',
				[ 'imeSubtypeExtraValue', 11 ],
				[ 'imeSubtypeLocale', 11 ],
				[ 'imeSubtypeMode', 11 ],
				[ 'immersive', 11 ],
				[ 'importantForAccessibility', 16 ],
				[ 'importantForAutofill', 26 ],
				[ 'importantForContentCapture', 30 ],
				'inAnimation',
				'includeFontPadding',
				'includeInGlobalSearch',
				'indeterminate',
				'indeterminateBehavior',
				'indeterminateDrawable',
				'indeterminateDuration',
				'indeterminateOnly',
				[ 'indeterminateProgressStyle', 11 ],
				[ 'indeterminateTint', 21 ],
				[ 'indeterminateTintMode', 21 ],
				[ 'indicatorEnd', 18 ],
				'indicatorLeft',
				'indicatorRight',
				[ 'indicatorStart', 18 ],
				'inflatedId',
				[ 'inheritShowWhenLocked', 29 ],
				'initOrder',
				[ 'initialKeyguardLayout', 17 ],
				'initialLayout',
				'innerRadius',
				'innerRadiusRatio',
				'inputMethod',
				'inputType',
				[ 'inset', 21 ],
				'insetBottom',
				'insetLeft',
				'insetRight',
				'insetTop',
				'installLocation',
				[ 'interactiveUiTimeout', 29 ],
				'interpolator',
				[ 'isAlwaysSyncable', 11 ],
				[ 'isAsciiCapable', 19 ],
				[ 'isAuxiliary', 14 ],
				'isDefault',
				[ 'isFeatureSplit', 26 ],
				[ 'isGame', 21 ],
				'isIndicator',
				[ 'isLightTheme', 29 ],
				'isModifier',
				'isRepeatable',
				'isScrollContainer',
				[ 'isSplitRequired', 29 ],
				[ 'isStatic', 26 ],
				'isSticky',
				[ 'isolatedProcess', 16 ],
				[ 'isolatedSplits', 26 ],
				'itemBackground',
				'itemIconDisabledAlpha',
				[ 'itemPadding', 11 ],
				'itemTextAppearance',
				[ 'justificationMode', 26 ],
				'keepScreenOn',
				'key',
				'keyBackground',
				'keyEdgeFlags',
				'keyHeight',
				'keyIcon',
				'keyLabel',
				'keyOutputText',
				'keyPreviewHeight',
				'keyPreviewLayout',
				'keyPreviewOffset',
				[ 'keySet', 19 ],
				'keyTextColor',
				'keyTextSize',
				'keyWidth',
				[ 'keyboardLayout', 16 ],
				'keyboardMode',
				[ 'keyboardNavigationCluster', 26 ],
				'keycode',
				'killAfterRestore',
				'label',
				[ 'labelFor', 17 ],
				'labelTextSize',
				[ 'languageTag', 24 ],
				[ 'largeHeap', 11 ],
				'largeScreens',
				[ 'largestWidthLimitDp', 13 ],
				[ 'lastBaselineToBottomHeight', 28 ],
				'launchMode',
				[ 'launchTaskBehindSourceAnimation', 21 ],
				[ 'launchTaskBehindTargetAnimation', 21 ],
				[ 'layerType', 11 ],
				'layout',
				'layoutAnimation',
				[ 'layoutDirection', 17 ],
				[ 'layoutMode', 18 ],
				'layout_above',
				'layout_alignBaseline',
				'layout_alignBottom',
				[ 'layout_alignEnd', 17 ],
				'layout_alignLeft',
				'layout_alignParentBottom',
				[ 'layout_alignParentEnd', 17 ],
				'layout_alignParentLeft',
				'layout_alignParentRight',
				[ 'layout_alignParentStart', 17 ],
				'layout_alignParentTop',
				'layout_alignRight',
				[ 'layout_alignStart', 17 ],
				'layout_alignTop',
				'layout_alignWithParentIfMissing',
				'layout_below',
				'layout_centerHorizontal',
				'layout_centerInParent',
				'layout_centerVertical',
				'layout_column',
				[ 'layout_columnSpan', 14 ],
				[ 'layout_columnWeight', 21 ],
				'layout_gravity',
				'layout_height',
				'layout_margin',
				'layout_marginBottom',
				[ 'layout_marginEnd', 17 ],
				[ 'layout_marginHorizontal', 26 ],
				'layout_marginLeft',
				'layout_marginRight',
				[ 'layout_marginStart', 17 ],
				'layout_marginTop',
				[ 'layout_marginVertical', 26 ],
				[ 'layout_row', 14 ],
				[ 'layout_rowSpan', 14 ],
				[ 'layout_rowWeight', 21 ],
				'layout_scale',
				'layout_span',
				[ 'layout_toEndOf', 17 ],
				'layout_toLeftOf',
				'layout_toRightOf',
				[ 'layout_toStartOf', 17 ],
				'layout_weight',
				'layout_width',
				'layout_x',
				'layout_y',
				'left',
				[ 'letterSpacing', 21 ],
				[ 'level', 24 ],
				[ 'lineHeight', 28 ],
				'lineSpacingExtra',
				'lineSpacingMultiplier',
				'lines',
				'linksClickable',
				[ 'listChoiceBackgroundIndicator', 11 ],
				'listChoiceIndicatorMultiple',
				'listChoiceIndicatorSingle',
				'listDivider',
				[ 'listDividerAlertDialog', 11 ],
				[ 'listMenuViewStyle', 24 ],
				[ 'listPopupWindowStyle', 11 ],
				'listPreferredItemHeight',
				[ 'listPreferredItemHeightLarge', 14 ],
				[ 'listPreferredItemHeightSmall', 14 ],
				[ 'listPreferredItemPaddingEnd', 17 ],
				[ 'listPreferredItemPaddingLeft', 14 ],
				[ 'listPreferredItemPaddingRight', 14 ],
				[ 'listPreferredItemPaddingStart', 17 ],
				'listSelector',
				'listSeparatorTextViewStyle',
				'listViewStyle',
				'listViewWhiteStyle',
				[ 'lockTaskMode', 23 ],
				[ 'logo', 11 ],
				[ 'logoDescription', 23 ],
				'longClickable',
				[ 'loopViews', 11 ],
				'manageSpaceActivity',
				'mapViewStyle',
				'marqueeRepeatLimit',
				[ 'matchOrder', 21 ],
				'max',
				[ 'maxAspectRatio', 26 ],
				[ 'maxButtonHeight', 24 ],
				[ 'maxDate', 11 ],
				'maxEms',
				'maxHeight',
				'maxItemsPerRow',
				'maxLength',
				'maxLevel',
				'maxLines',
				[ 'maxLongVersionCode', 28 ],
				[ 'maxRecents', 21 ],
				'maxRows',
				'maxSdkVersion',
				'maxWidth',
				[ 'maximumAngle', 21 ],
				'measureAllChildren',
				[ 'measureWithLargestChild', 11 ],
				[ 'mediaRouteButtonStyle', 16 ],
				[ 'mediaRouteTypes', 16 ],
				'menuCategory',
				[ 'mimeGroup', 30 ],
				'mimeType',
				[ 'min', 26 ],
				[ 'minAspectRatio', 29 ],
				[ 'minDate', 11 ],
				'minEms',
				'minHeight',
				'minLevel',
				'minLines',
				[ 'minResizeHeight', 14 ],
				[ 'minResizeWidth', 14 ],
				'minSdkVersion',
				'minWidth',
				[ 'minimumHorizontalAngle', 21 ],
				[ 'minimumVerticalAngle', 21 ],
				[ 'mipMap', 18 ],
				[ 'mirrorForRtl', 18 ],
				'mode',
				'moreIcon',
				[ 'multiArch', 21 ],
				'multiprocess',
				'name',
				[ 'navigationBarColor', 21 ],
				[ 'navigationBarDividerColor', 27 ],
				[ 'navigationContentDescription', 21 ],
				[ 'navigationIcon', 21 ],
				[ 'navigationMode', 11 ],
				'negativeButtonText',
				[ 'nestedScrollingEnabled', 21 ],
				[ 'networkSecurityConfig', 24 ],
				[ 'nextClusterForward', 26 ],
				'nextFocusDown',
				[ 'nextFocusForward', 11 ],
				'nextFocusLeft',
				'nextFocusRight',
				'nextFocusUp',
				'noHistory',
				[ 'nonInteractiveUiTimeout', 29 ],
				'normalScreens',
				[ 'notificationTimeout', 14 ],
				'numColumns',
				'numStars',
				[ 'numberPickerStyle', 24 ],
				[ 'numbersBackgroundColor', 21 ],
				[ 'numbersInnerTextColor', 23 ],
				[ 'numbersSelectorColor', 21 ],
				[ 'numbersTextColor', 21 ],
				'numeric',
				[ 'numericModifiers', 26 ],
				'numericShortcut',
				[ 'offset', 24 ],
				'onClick',
				'oneshot',
				[ 'opacity', 11 ],
				[ 'opticalInsetBottom', 29 ],
				[ 'opticalInsetLeft', 29 ],
				[ 'opticalInsetRight', 29 ],
				[ 'opticalInsetTop', 29 ],
				'order',
				'orderInCategory',
				[ 'ordering', 11 ],
				'orderingFromXml',
				'orientation',
				'outAnimation',
				[ 'outlineAmbientShadowColor', 28 ],
				[ 'outlineProvider', 21 ],
				[ 'outlineSpotShadowColor', 28 ],
				'overScrollFooter',
				'overScrollHeader',
				'overScrollMode',
				[ 'overlapAnchor', 21 ],
				[ 'overridesImplicitlyEnabledSubtype', 14 ],
				[ 'packageNames', 14 ],
				[ 'packageType', 29 ],
				'padding',
				'paddingBottom',
				[ 'paddingEnd', 17 ],
				[ 'paddingHorizontal', 26 ],
				'paddingLeft',
				[ 'paddingMode', 21 ],
				'paddingRight',
				[ 'paddingStart', 17 ],
				'paddingTop',
				[ 'paddingVertical', 26 ],
				'panelBackground',
				'panelColorBackground',
				'panelColorForeground',
				'panelFullBackground',
				'panelTextAppearance',
				[ 'parentActivityName', 16 ],
				'password',
				'path',
				[ 'pathData', 21 ],
				'pathPattern',
				'pathPrefix',
				[ 'patternPathData', 21 ],
				'permission',
				[ 'permissionFlags', 17 ],
				'permissionGroup',
				[ 'permissionGroupFlags', 17 ],
				[ 'persistableMode', 21 ],
				'persistent',
				'persistentDrawingCache',
				[ 'persistentWhenFeatureAvailable', 26 ],
				'phoneNumber',
				'pivotX',
				'pivotY',
				[ 'pointerIcon', 24 ],
				'popupAnimationStyle',
				'popupBackground',
				'popupCharacters',
				[ 'popupElevation', 21 ],
				[ 'popupEnterTransition', 24 ],
				[ 'popupExitTransition', 24 ],
				'popupKeyboard',
				'popupLayout',
				[ 'popupMenuStyle', 11 ],
				[ 'popupTheme', 21 ],
				'popupWindowStyle',
				'port',
				'positiveButtonText',
				[ 'preferMinimalPostProcessing', 30 ],
				'preferenceCategoryStyle',
				[ 'preferenceFragmentStyle', 24 ],
				'preferenceInformationStyle',
				'preferenceLayoutChild',
				'preferenceScreenStyle',
				'preferenceStyle',
				[ 'presentationTheme', 17 ],
				[ 'preserveLegacyExternalStorage', 30 ],
				[ 'previewImage', 11 ],
				[ 'primaryContentAlpha', 26 ],
				'priority',
				'privateImeOptions',
				'process',
				'progress',
				[ 'progressBackgroundTint', 21 ],
				[ 'progressBackgroundTintMode', 21 ],
				[ 'progressBarPadding', 11 ],
				'progressBarStyle',
				'progressBarStyleHorizontal',
				'progressBarStyleInverse',
				'progressBarStyleLarge',
				'progressBarStyleLargeInverse',
				'progressBarStyleSmall',
				'progressBarStyleSmallInverse',
				'progressBarStyleSmallTitle',
				'progressDrawable',
				[ 'progressTint', 21 ],
				[ 'progressTintMode', 21 ],
				'prompt',
				[ 'propertyName', 11 ],
				[ 'propertyXName', 21 ],
				[ 'propertyYName', 21 ],
				'protectionLevel',
				[ 'publicKey', 14 ],
				'queryActionMsg',
				'queryAfterZeroResults',
				[ 'queryBackground', 21 ],
				[ 'queryHint', 11 ],
				'quickContactBadgeStyleSmallWindowLarge',
				'quickContactBadgeStyleSmallWindowMedium',
				'quickContactBadgeStyleSmallWindowSmall',
				'quickContactBadgeStyleWindowLarge',
				'quickContactBadgeStyleWindowMedium',
				'quickContactBadgeStyleWindowSmall',
				'radioButtonStyle',
				'radius',
				'rating',
				'ratingBarStyle',
				'ratingBarStyleIndicator',
				'ratingBarStyleSmall',
				'readPermission',
				[ 'recognitionService', 21 ],
				[ 'recreateOnConfigChanges', 26 ],
				[ 'recycleEnabled', 26 ],
				[ 'relinquishTaskIdentity', 21 ],
				[ 'reparent', 21 ],
				[ 'reparentWithOverlay', 21 ],
				'repeatCount',
				'repeatMode',
				'reqFiveWayNav',
				'reqHardKeyboard',
				'reqKeyboardType',
				'reqNavigation',
				'reqTouchScreen',
				[ 'requestLegacyExternalStorage', 29 ],
				[ 'requireDeviceUnlock', 19 ],
				'required',
				[ 'requiredAccountType', 18 ],
				[ 'requiredFeature', 26 ],
				[ 'requiredForAllUsers', 18 ],
				[ 'requiredNotFeature', 26 ],
				[ 'requiresFadingEdge', 14 ],
				[ 'requiresSmallestWidthDp', 13 ],
				[ 'resizeClip', 22 ],
				[ 'resizeMode', 12 ],
				'resizeable',
				[ 'resizeableActivity', 24 ],
				'resource',
				[ 'resourcesMap', 30 ],
				'restoreAnyVersion',
				'restoreNeedsApplication',
				[ 'restrictedAccountType', 18 ],
				[ 'restrictionType', 21 ],
				[ 'resumeWhilePausing', 21 ],
				[ 'reversible', 21 ],
				[ 'revisionCode', 22 ],
				'right',
				'ringtonePreferenceStyle',
				'ringtoneType',
				[ 'rotation', 11 ],
				[ 'rotationAnimation', 26 ],
				[ 'rotationX', 11 ],
				[ 'rotationY', 11 ],
				[ 'roundIcon', 25 ],
				[ 'rowCount', 14 ],
				'rowDelay',
				'rowEdgeFlags',
				'rowHeight',
				[ 'rowOrderPreserved', 14 ],
				'saveEnabled',
				'scaleGravity',
				'scaleHeight',
				'scaleType',
				'scaleWidth',
				[ 'scaleX', 11 ],
				[ 'scaleY', 11 ],
				'scheme',
				'screenDensity',
				'screenOrientation',
				[ 'screenReaderFocusable', 28 ],
				'screenSize',
				'scrollHorizontally',
				[ 'scrollIndicators', 23 ],
				'scrollViewStyle',
				'scrollX',
				'scrollY',
				'scrollbarAlwaysDrawHorizontalTrack',
				'scrollbarAlwaysDrawVerticalTrack',
				'scrollbarDefaultDelayBeforeFade',
				'scrollbarFadeDuration',
				'scrollbarSize',
				'scrollbarStyle',
				'scrollbarThumbHorizontal',
				'scrollbarThumbVertical',
				'scrollbarTrackHorizontal',
				'scrollbarTrackVertical',
				'scrollbars',
				'scrollingCache',
				'searchButtonText',
				[ 'searchHintIcon', 22 ],
				[ 'searchIcon', 21 ],
				'searchMode',
				'searchSettingsDescription',
				'searchSuggestAuthority',
				'searchSuggestIntentAction',
				'searchSuggestIntentData',
				'searchSuggestPath',
				'searchSuggestSelection',
				'searchSuggestThreshold',
				[ 'searchViewStyle', 21 ],
				[ 'secondaryContentAlpha', 26 ],
				'secondaryProgress',
				[ 'secondaryProgressTint', 21 ],
				[ 'secondaryProgressTintMode', 21 ],
				[ 'secureElementName', 29 ],
				'seekBarStyle',
				[ 'segmentedButtonStyle', 11 ],
				'selectAllOnFocus',
				'selectable',
				[ 'selectableItemBackground', 11 ],
				[ 'selectableItemBackgroundBorderless', 21 ],
				[ 'selectedDateVerticalBar', 11 ],
				[ 'selectedWeekBackgroundColor', 11 ],
				[ 'selectionDividerHeight', 29 ],
				[ 'sessionService', 21 ],
				'settingsActivity',
				[ 'settingsSliceUri', 29 ],
				[ 'setupActivity', 21 ],
				'shadowColor',
				'shadowDx',
				'shadowDy',
				'shadowRadius',
				'shape',
				'shareInterpolator',
				'sharedUserId',
				'sharedUserLabel',
				[ 'shell', 29 ],
				[ 'shortcutDisabledMessage', 25 ],
				[ 'shortcutId', 25 ],
				[ 'shortcutLongLabel', 25 ],
				[ 'shortcutShortLabel', 25 ],
				'shouldDisableView',
				[ 'showAsAction', 11 ],
				'showDefault',
				[ 'showDividers', 11 ],
				[ 'showForAllUsers', 23 ],
				[ 'showMetadataInPreview', 25 ],
				[ 'showOnLockScreen', 17 ],
				'showSilent',
				[ 'showText', 21 ],
				[ 'showWeekNumber', 11 ],
				[ 'showWhenLocked', 27 ],
				[ 'shownWeekCount', 11 ],
				'shrinkColumns',
				'singleLine',
				[ 'singleLineTitle', 26 ],
				[ 'singleUser', 17 ],
				[ 'slideEdge', 21 ],
				'smallIcon',
				'smallScreens',
				'smoothScrollbar',
				'soundEffectsEnabled',
				'spacing',
				'spinnerDropDownItemStyle',
				'spinnerItemStyle',
				[ 'spinnerMode', 11 ],
				'spinnerStyle',
				[ 'spinnersShown', 11 ],
				[ 'splitMotionEvents', 11 ],
				[ 'splitName', 26 ],
				[ 'splitTrack', 21 ],
				[ 'spotShadowAlpha', 21 ],
				'src',
				[ 'ssp', 19 ],
				[ 'sspPattern', 19 ],
				[ 'sspPrefix', 19 ],
				'stackFromBottom',
				[ 'stackViewStyle', 21 ],
				'starStyle',
				[ 'start', 23 ],
				'startColor',
				[ 'startDelay', 19 ],
				'startOffset',
				[ 'startX', 24 ],
				[ 'startY', 24 ],
				'startYear',
				[ 'stateListAnimator', 21 ],
				'stateNotNeeded',
				'state_above_anchor',
				[ 'state_accelerated', 11 ],
				[ 'state_activated', 11 ],
				'state_active',
				'state_checkable',
				'state_checked',
				[ 'state_drag_can_accept', 14 ],
				[ 'state_drag_hovered', 14 ],
				'state_empty',
				'state_enabled',
				'state_expanded',
				'state_first',
				'state_focused',
				[ 'state_hovered', 14 ],
				'state_last',
				'state_long_pressable',
				'state_middle',
				[ 'state_multiline', 11 ],
				'state_pressed',
				'state_selected',
				'state_single',
				'state_window_focused',
				[ 'staticWallpaperPreview', 11 ],
				[ 'statusBarColor', 21 ],
				'stepSize',
				[ 'stopWithTask', 14 ],
				'streamType',
				'stretchColumns',
				'stretchMode',
				[ 'strokeAlpha', 21 ],
				[ 'strokeColor', 21 ],
				[ 'strokeLineCap', 21 ],
				[ 'strokeLineJoin', 21 ],
				[ 'strokeMiterLimit', 21 ],
				[ 'strokeWidth', 21 ],
				[ 'subMenuArrow', 24 ],
				[ 'submitBackground', 21 ],
				[ 'subtitle', 11 ],
				[ 'subtitleTextAppearance', 21 ],
				[ 'subtitleTextColor', 23 ],
				[ 'subtitleTextStyle', 11 ],
				[ 'subtypeExtraValue', 14 ],
				[ 'subtypeId', 17 ],
				[ 'subtypeLocale', 14 ],
				'suggestActionMsg',
				'suggestActionMsgColumn',
				[ 'suggestionRowLayout', 21 ],
				'summary',
				'summaryColumn',
				'summaryOff',
				'summaryOn',
				[ 'supportsAssist', 23 ],
				[ 'supportsInlineSuggestions', 30 ],
				[ 'supportsLaunchVoiceAssistFromKeyguard', 23 ],
				[ 'supportsLocalInteraction', 24 ],
				[ 'supportsMultipleDisplays', 29 ],
				[ 'supportsPictureInPicture', 24 ],
				[ 'supportsRtl', 17 ],
				[ 'supportsSwitchingToNextInputMethod', 19 ],
				'supportsUploading',
				[ 'switchMinWidth', 14 ],
				[ 'switchPadding', 14 ],
				[ 'switchPreferenceStyle', 14 ],
				[ 'switchStyle', 21 ],
				[ 'switchTextAppearance', 14 ],
				[ 'switchTextOff', 14 ],
				[ 'switchTextOn', 14 ],
				'syncable',
				'tabStripEnabled',
				'tabStripLeft',
				'tabStripRight',
				'tabWidgetStyle',
				'tag',
				'targetActivity',
				'targetClass',
				[ 'targetDescriptions', 14 ],
				[ 'targetId', 19 ],
				[ 'targetName', 21 ],
				'targetPackage',
				[ 'targetProcesses', 26 ],
				[ 'targetSandboxVersion', 26 ],
				'targetSdkVersion',
				'taskAffinity',
				'taskCloseEnterAnimation',
				'taskCloseExitAnimation',
				'taskOpenEnterAnimation',
				'taskOpenExitAnimation',
				'taskToBackEnterAnimation',
				'taskToBackExitAnimation',
				'taskToFrontEnterAnimation',
				'taskToFrontExitAnimation',
				'tension',
				'testOnly',
				'text',
				[ 'textAlignment', 17 ],
				[ 'textAllCaps', 14 ],
				'textAppearance',
				'textAppearanceButton',
				'textAppearanceInverse',
				'textAppearanceLarge',
				'textAppearanceLargeInverse',
				[ 'textAppearanceLargePopupMenu', 11 ],
				[ 'textAppearanceListItem', 14 ],
				[ 'textAppearanceListItemSecondary', 21 ],
				[ 'textAppearanceListItemSmall', 14 ],
				'textAppearanceMedium',
				'textAppearanceMediumInverse',
				[ 'textAppearancePopupMenuHeader', 24 ],
				'textAppearanceSearchResultSubtitle',
				'textAppearanceSearchResultTitle',
				'textAppearanceSmall',
				'textAppearanceSmallInverse',
				[ 'textAppearanceSmallPopupMenu', 11 ],
				'textCheckMark',
				'textCheckMarkInverse',
				'textColor',
				[ 'textColorAlertDialogListItem', 11 ],
				'textColorHighlight',
				[ 'textColorHighlightInverse', 11 ],
				'textColorHint',
				'textColorHintInverse',
				'textColorLink',
				[ 'textColorLinkInverse', 11 ],
				'textColorPrimary',
				'textColorPrimaryDisableOnly',
				'textColorPrimaryInverse',
				'textColorPrimaryInverseDisableOnly',
				'textColorPrimaryInverseNoDisable',
				'textColorPrimaryNoDisable',
				'textColorSecondary',
				'textColorSecondaryInverse',
				'textColorSecondaryInverseNoDisable',
				'textColorSecondaryNoDisable',
				'textColorTertiary',
				'textColorTertiaryInverse',
				[ 'textCursorDrawable', 12 ],
				[ 'textDirection', 17 ],
				[ 'textEditNoPasteWindowLayout', 11 ],
				[ 'textEditPasteWindowLayout', 11 ],
				[ 'textEditSideNoPasteWindowLayout', 11 ],
				[ 'textEditSidePasteWindowLayout', 11 ],
				[ 'textEditSuggestionItemLayout', 14 ],
				'textFilterEnabled',
				[ 'textFontWeight', 28 ],
				[ 'textIsSelectable', 11 ],
				[ 'textLocale', 29 ],
				'textOff',
				'textOn',
				'textScaleX',
				'textSelectHandle',
				'textSelectHandleLeft',
				'textSelectHandleRight',
				'textSelectHandleWindowStyle',
				'textSize',
				'textStyle',
				[ 'textSuggestionsWindowStyle', 14 ],
				'textViewStyle',
				'theme',
				'thickness',
				'thicknessRatio',
				'thumb',
				'thumbOffset',
				[ 'thumbPosition', 23 ],
				[ 'thumbTextPadding', 114 ],
				[ 'thumbTint', 21 ],
				[ 'thumbTintMode', 21 ],
				'thumbnail',
				[ 'tickMark', 24 ],
				[ 'tickMarkTint', 24 ],
				[ 'tickMarkTintMode', 24 ],
				'tileMode',
				[ 'tileModeX', 21 ],
				[ 'tileModeY', 21 ],
				[ 'timePickerDialogTheme', 21 ],
				[ 'timePickerMode', 21 ],
				[ 'timePickerStyle', 21 ],
				[ 'timeZone', 17 ],
				'tint',
				[ 'tintMode', 21 ],
				'title',
				'titleCondensed',
				[ 'titleMargin', 24 ],
				[ 'titleMarginBottom', 24 ],
				[ 'titleMarginEnd', 24 ],
				[ 'titleMarginStart', 24 ],
				[ 'titleMarginTop', 24 ],
				[ 'titleTextAppearance', 21 ],
				[ 'titleTextColor', 23 ],
				[ 'titleTextStyle', 11 ],
				'toAlpha',
				'toDegrees',
				[ 'toId', 21 ],
				[ 'toScene', 19 ],
				'toXDelta',
				'toXScale',
				'toYDelta',
				'toYScale',
				[ 'toolbarStyle', 21 ],
				[ 'tooltipText', 26 ],
				'top',
				'topBright',
				'topDark',
				'topLeftRadius',
				'topOffset',
				'topRightRadius',
				[ 'touchscreenBlocksFocus', 21 ],
				[ 'track', 14 ],
				[ 'trackTint', 23 ],
				[ 'trackTintMode', 23 ],
				'transcriptMode',
				[ 'transformPivotX', 11 ],
				[ 'transformPivotY', 11 ],
				[ 'transition', 19 ],
				[ 'transitionGroup', 21 ],
				[ 'transitionName', 21 ],
				[ 'transitionOrdering', 19 ],
				[ 'transitionVisibilityMode', 21 ],
				[ 'translateX', 21 ],
				[ 'translateY', 21 ],
				[ 'translationX', 11 ],
				[ 'translationY', 11 ],
				[ 'translationZ', 21 ],
				[ 'trimPathEnd', 21 ],
				[ 'trimPathOffset', 21 ],
				[ 'trimPathStart', 21 ],
				[ 'ttcIndex', 28 ],
				[ 'tunerCount', 24 ],
				[ 'turnScreenOn', 27 ],
				'type',
				'typeface',
				[ 'uiOptions', 14 ],
				'uncertainGestureColor',
				[ 'unfocusedMonthDateColor', 11 ],
				'unselectedAlpha',
				'updatePeriodMillis',
				[ 'use32bitAbi', 24 ],
				[ 'useAppZygote', 29 ],
				[ 'useDefaultMargins', 14 ],
				[ 'useEmbeddedDex', 29 ],
				[ 'useIntrinsicSizeAsMinimum', 11 ],
				'useLevel',
				'userVisible',
				[ 'usesCleartextTraffic', 23 ],
				'value',
				[ 'valueFrom', 11 ],
				[ 'valueTo', 11 ],
				[ 'valueType', 11 ],
				'variablePadding',
				[ 'vendor', 19 ],
				[ 'version', 24 ],
				'versionCode',
				[ 'versionCodeMajor', 28 ],
				[ 'versionMajor', 28 ],
				'versionName',
				'verticalCorrection',
				'verticalDivider',
				'verticalGap',
				[ 'verticalScrollbarPosition', 11 ],
				'verticalSpacing',
				[ 'viewportHeight', 21 ],
				[ 'viewportWidth', 21 ],
				'visibility',
				'visible',
				[ 'visibleToInstantApps', 26 ],
				'vmSafeMode',
				[ 'voiceIcon', 21 ],
				'voiceLanguage',
				'voiceLanguageModel',
				'voiceMaxResults',
				'voicePromptText',
				'voiceSearchMode',
				'wallpaperCloseEnterAnimation',
				'wallpaperCloseExitAnimation',
				'wallpaperIntraCloseEnterAnimation',
				'wallpaperIntraCloseExitAnimation',
				'wallpaperIntraOpenEnterAnimation',
				'wallpaperIntraOpenExitAnimation',
				'wallpaperOpenEnterAnimation',
				'wallpaperOpenExitAnimation',
				'webTextViewStyle',
				'webViewStyle',
				[ 'weekDayTextAppearance', 11 ],
				[ 'weekNumberColor', 11 ],
				[ 'weekSeparatorLineColor', 11 ],
				'weightSum',
				[ 'widgetCategory', 17 ],
				[ 'widgetFeatures', 28 ],
				'widgetLayout',
				'width',
				[ 'windowActionBar', 11 ],
				[ 'windowActionBarOverlay', 11 ],
				[ 'windowActionModeOverlay', 11 ],
				[ 'windowActivityTransitions', 21 ],
				[ 'windowAllowEnterTransitionOverlap', 21 ],
				[ 'windowAllowReturnTransitionOverlap', 21 ],
				'windowAnimationStyle',
				'windowBackground',
				[ 'windowBackgroundFallback', 24 ],
				[ 'windowClipToOutline', 21 ],
				[ 'windowCloseOnTouchOutside', 11 ],
				'windowContentOverlay',
				[ 'windowContentTransitionManager', 21 ],
				[ 'windowContentTransitions', 21 ],
				'windowDisablePreview',
				[ 'windowDrawsSystemBarBackgrounds', 21 ],
				[ 'windowElevation', 21 ],
				[ 'windowEnableSplitTouch', 11 ],
				'windowEnterAnimation',
				[ 'windowEnterTransition', 21 ],
				'windowExitAnimation',
				[ 'windowExitTransition', 21 ],
				'windowFrame',
				'windowFullscreen',
				'windowHideAnimation',
				'windowIsFloating',
				'windowIsTranslucent',
				[ 'windowLayoutInDisplayCutoutMode', 27 ],
				[ 'windowLightNavigationBar', 27 ],
				[ 'windowLightStatusBar', 23 ],
				[ 'windowMinWidthMajor', 11 ],
				[ 'windowMinWidthMinor', 11 ],
				'windowNoDisplay',
				'windowNoTitle',
				[ 'windowOverscan', 18 ],
				[ 'windowReenterTransition', 21 ],
				[ 'windowReturnTransition', 21 ],
				[ 'windowSharedElementEnterTransition', 21 ],
				[ 'windowSharedElementExitTransition', 21 ],
				[ 'windowSharedElementReenterTransition', 21 ],
				[ 'windowSharedElementReturnTransition', 21 ],
				[ 'windowSharedElementsUseOverlay', 21 ],
				'windowShowAnimation',
				'windowShowWallpaper',
				'windowSoftInputMode',
				[ 'windowSplashscreenContent', 26 ],
				[ 'windowSwipeToDismiss', 20 ],
				'windowTitleBackgroundStyle',
				'windowTitleSize',
				'windowTitleStyle',
				[ 'windowTransitionBackgroundFadeDuration', 21 ],
				[ 'windowTranslucentNavigation', 19 ],
				[ 'windowTranslucentStatus', 19 ],
				'writePermission',
				'x',
				'xlargeScreens',
				'y',
				[ 'yearListItemTextAppearance', 21 ],
				[ 'yearListSelectorColor', 21 ],
				'yesNoPreferenceStyle',
				'zAdjustment',
				[ 'zygotePreloadName', 29 ],
			];
			for (let prop of properties) {
				if (Array.isArray(prop)) {
					const apiLevel = prop[1];
					prop = prop[0];
					if (Ti.Platform.Android.API_LEVEL < apiLevel) {
						continue; // skip
					}
				}
				it(`.${prop} is a Number`, () => {
					should(Ti.Android.R.attr[prop]).be.a.Number();
					should(Ti.Android.R.attr[prop]).not.eql(0);
				});
			}
		});

		describe('.bool', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('bool').which.is.a.Object();
			});
		});

		describe('.color', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('color').which.is.a.Object();
			});

			const properties = [
				'background_dark',
				'background_light',
				'black',
				'darker_gray',
				'holo_blue_bright',
				'holo_blue_dark',
				'holo_blue_light',
				'holo_green_dark',
				'holo_green_light',
				'holo_orange_dark',
				'holo_orange_light',
				'holo_purple',
				'holo_red_dark',
				'holo_red_light',
				'primary_text_dark',
				'primary_text_dark_nodisable',
				'primary_text_light',
				'primary_text_light_nodisable',
				'secondary_text_dark',
				'secondary_text_dark_nodisable',
				'secondary_text_light',
				'secondary_text_light_nodisable',
				'tab_indicator_text',
				'tertiary_text_dark',
				'tertiary_text_light',
				'transparent',
				'white',
				'widget_edittext_dark',
			];
			for (const prop of properties) {
				it(`.${prop} is a Number`, () => {
					should(Ti.Android.R.color[prop]).be.a.Number();
					should(Ti.Android.R.color[prop]).not.eql(0);
				});
			}
		});

		describe('.dimen', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('dimen').which.is.a.Object();
			});

			const properties = [
				'app_icon_size',
				'dialog_min_width_major',
				'dialog_min_width_minor',
				'notification_large_icon_height',
				'notification_large_icon_width',
				'thumbnail_height',
				'thumbnail_width',
			];
			for (const prop of properties) {
				it(`.${prop} is a Number`, () => {
					should(Ti.Android.R.dimen[prop]).be.a.Number();
					should(Ti.Android.R.dimen[prop]).not.eql(0);
				});
			}
		});

		describe('.drawable', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('drawable').which.is.a.Object();
			});

			// TODO: Test known properties
		});

		describe('.fraction', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('fraction').which.is.a.Object();
			});
		});

		describe('.id', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('id').which.is.a.Object();
			});

			// TODO: Test known properties
		});

		describe('.integer', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('integer').which.is.a.Object();
			});

			const properties = [
				'config_longAnimTime',
				'config_mediumAnimTime',
				'config_shortAnimTime',
				'status_bar_notification_info_maxnum',
			];
			for (const prop of properties) {
				it(`.${prop} is a Number`, () => {
					should(Ti.Android.R.integer[prop]).be.a.Number();
					should(Ti.Android.R.integer[prop]).not.eql(0);
				});
			}
		});

		describe('.interpolator', () => {
			// Only supported on Android API 11+
			before(function () {
				if (Ti.Platform.Android.API_LEVEL < 11) {
					this.skip();
				}
			});

			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('interpolator').which.is.a.Object();
			});

			const properties = [
				'accelerate_cubic',
				'accelerate_decelerate',
				'accelerate_quad',
				'accelerate_quint',
				'anticipate',
				'anticipate_overshoot',
				'bounce',
				'cycle',
				'decelerate_cubic',
				'decelerate_quad',
				'decelerate_quint',
				[ 'fast_out_extra_slow_in', 28 ],
				[ 'fast_out_linear_in', 21 ],
				[ 'fast_out_slow_in', 21 ],
				'linear',
				[ 'linear_out_slow_in', 21 ],
				'overshoot',
			];
			for (let prop of properties) {
				if (Array.isArray(prop)) {
					const apiLevel = prop[1];
					prop = prop[0];
					if (Ti.Platform.Android.API_LEVEL < apiLevel) {
						continue; // skip
					}
				}
				it(`.${prop} is a Number`, () => {
					should(Ti.Android.R.interpolator[prop]).be.a.Number();
					should(Ti.Android.R.interpolator[prop]).not.eql(0);
				});
			}
		});

		describe('.layout', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('layout').which.is.a.Object();
			});

			const properties = [
				'activity_list_item',
				'browser_link_context_header',
				'expandable_list_content',
				'list_content',
				'preference_category',
				'select_dialog_item',
				'select_dialog_multichoice',
				'select_dialog_singlechoice',
				'simple_dropdown_item_1line',
				'simple_expandable_list_item_1',
				'simple_expandable_list_item_2',
				'simple_gallery_item',
				'simple_list_item_1',
				'simple_list_item_2',
				'simple_list_item_activated_1',
				'simple_list_item_activated_2',
				'simple_list_item_checked',
				'simple_list_item_multiple_choice',
				'simple_list_item_single_choice',
				'simple_selectable_list_item',
				'simple_spinner_dropdown_item',
				'simple_spinner_item',
				'test_list_item',
				'two_line_list_item',
			];
			for (const prop of properties) {
				it(`.${prop} is a Number`, () => {
					should(Ti.Android.R.layout[prop]).be.a.Number();
					should(Ti.Android.R.layout[prop]).not.eql(0);
				});
			}
		});

		describe('.menu', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('menu').which.is.a.Object();
			});
		});

		describe('.mipmap', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('mipmap').which.is.a.Object();
			});

			const properties = [
				'sym_def_app_icon',
			];
			for (const prop of properties) {
				it(`.${prop} is a Number`, () => {
					should(Ti.Android.R.mipmap[prop]).be.a.Number();
					should(Ti.Android.R.mipmap[prop]).not.eql(0);
				});
			}
		});

		describe('.plurals', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('plurals').which.is.a.Object();
			});
		});

		describe('.raw', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('raw').which.is.a.Object();
			});
		});

		describe('.string', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('string').which.is.a.Object();
			});

			const properties = [
				'VideoView_error_button',
				'VideoView_error_text_invalid_progressive_playback',
				'VideoView_error_text_unknown',
				'VideoView_error_title',
				[ 'autofill', 27 ],
				'cancel',
				'copy',
				'copyUrl',
				'cut',
				'defaultMsisdnAlphaTag',
				'defaultVoiceMailAlphaTag',
				'dialog_alert_title',
				'emptyPhoneNumber',
				[ 'fingerprint_icon_content_description', 23 ],
				'httpErrorBadUrl',
				'httpErrorUnsupportedScheme',
				'no',
				'ok',
				'paste',
				[ 'paste_as_plain_text', 26 ],
				'search_go',
				'selectAll',
				'selectTextMode',
				[ 'status_bar_notification_info_overflow', 14 ],
				'unknownName',
				'untitled',
				'yes',
			];
			for (let prop of properties) {
				if (Array.isArray(prop)) {
					const apiLevel = prop[1];
					prop = prop[0];
					if (Ti.Platform.Android.API_LEVEL < apiLevel) {
						continue; // skip
					}
				}
				it(`.${prop} is a Number`, () => {
					should(Ti.Android.R.string[prop]).be.a.Number();
					should(Ti.Android.R.string[prop]).not.eql(0);
				});
			}
		});

		describe('.style', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('style').which.is.a.Object();
			});

			// TODO: Test for known properties
		});

		describe('.styleable', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('styleable').which.is.a.Object();
			});

			// TODO: Test for known properties
		});

		describe('.transition', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('transition').which.is.a.Object();
			});

			const properties = [
				'explode',
				'fade',
				'move',
				'no_transition',
				'slide_bottom',
				'slide_left',
				'slide_right',
				'slide_top',
			];
			for (const prop of properties) {
				it(`.${prop} is a Number`, () => {
					should(Ti.Android.R.transition[prop]).be.a.Number();
					should(Ti.Android.R.transition[prop]).not.eql(0);
				});
			}
		});

		describe('.xml', () => {
			it('is a Object', () => {
				should(Ti.Android.R).have.a.readOnlyProperty('xml').which.is.a.Object();
			});
		});

	});
});
